@@ -113,6 +113,7 @@ class Runner
       @basepath + 'src/backend/jit/llvm/llvmjit_expr.c', # Requires LLVM-C library (which we don't want to require)
       @basepath + 'src/backend/jit/llvm/llvmjit_deform.c', # Requires LLVM-C library (which we don't want to require)
       @basepath + 'src/backend/jit/llvm/llvmjit.c', # Requires LLVM-C library (which we don't want to require)
+      @basepath + 'src/backend/libpq/be-gssapi-common.c', # Requires GSSAPI (which we don't want to require)
       @basepath + 'src/backend/libpq/be-secure-gssapi.c', # Requires GSSAPI (which we don't want to require)
       @basepath + 'src/common/protocol_openssl.c', # Requires OpenSSL (which we don't want to require)
     ] -
@@ -211,7 +212,14 @@ class Runner
 
   def analyze_file(file)
     index = FFI::Clang::Index.new(true, true)
-    translation_unit = index.parse_translation_unit(file, ['-I', @basepath + 'src/include', '-I', '/usr/local/opt/openssl/include', '-I', `xcrun --sdk macosx --show-sdk-path`.strip + '/usr/include', '-DDLSUFFIX=".bundle"', '-msse4.2', '-g', '-DUSE_ASSERT_CHECKING'])
+    translation_unit = index.parse_translation_unit(file, [
+      '-I', @basepath + 'src/include',
+      '-I', '/usr/local/opt/openssl/include',
+      '-I', `xcrun --sdk macosx --show-sdk-path`.strip + '/usr/include',
+      '-DDLSUFFIX=".bundle"',
+      '-g',
+      '-DUSE_ASSERT_CHECKING'
+    ])
     cursor = translation_unit.cursor
 
     func_cursor = nil
@@ -253,10 +261,15 @@ class Runner
             analysis.file_to_symbol_positions[cursor.location.file][cursor.spelling] = [start_offset, end_offset]
 
             cursor.visit_children do |child_cursor, parent|
+              # There seems to be a bug here on modern Clang versions where the
+              # cursor kind gets modified once we call "child_cursor.definition"
+              # - thus we make a copy ahead of calling that, for later use
+              child_cursor_kind = child_cursor.kind
+
               # Ignore variable definitions from the local scope
               next :recurse if child_cursor.definition.semantic_parent == cursor
 
-              if child_cursor.kind == :cursor_decl_ref_expr || child_cursor.kind == :cursor_call_expr
+              if child_cursor_kind == :cursor_decl_ref_expr || child_cursor_kind == :cursor_call_expr
                 analysis.references[cursor.spelling] ||= []
                 (analysis.references[cursor.spelling] << child_cursor.spelling).uniq!
               end
@@ -399,7 +412,7 @@ class Runner
         else
           # In the off chance that part of a macro is before a symbol (e.g. ifdef),
           # but the closing part is inside (e.g. endif) we need to output all macros inside skipped parts
-          str += "\n" + skipped_code.scan(/^(#\s*(?:include|define|undef|if|ifdef|ifndef|else|endif))((?:[^\n]*\\\s*\n)*)([^\n]*)$/m).map { |m| m.compact.join }.join("\n")
+          str += "\n" + skipped_code.scan(/^(#\s*(?:define|undef|if|ifdef|ifndef|else|endif))((?:[^\n]*\\\s*\n)*)([^\n]*)$/m).map { |m| m.compact.join }.join("\n")
         end
 
         next_start_pos = pos[1]
@@ -539,6 +552,7 @@ runner.deep_resolve('pg_toupper')
 
 # Needed for normalize
 runner.deep_resolve('pg_qsort')
+runner.deep_resolve('pg_qsort_strcmp')
 runner.deep_resolve('raw_expression_tree_walker')
 
 # Needed to work with simplehash (in fingerprinting logic)
