@@ -11,7 +11,7 @@ const char* tests[] = {
   "SELECT * INTO films_recent FROM films WHERE date_prod >= '2002-01-01'",
   "SELECT current_timestamp",
   "SELECT current_time(2)",
-  "SELECT memory_total_bytes, memory_swap_total_bytes - memory_swap_free_bytes AS swap, date_part(?, s.collected_at) AS collected_at FROM snapshots s JOIN system_snapshots ON snapshot_id = s.id WHERE s.database_id = ? AND s.collected_at >= ? AND s.collected_at <= ? ORDER BY collected_at ASC",
+  "SELECT memory_total_bytes, memory_swap_total_bytes - memory_swap_free_bytes AS swap, date_part('year', s.collected_at) AS collected_at FROM snapshots s JOIN system_snapshots ON snapshot_id = s.id WHERE s.database_id = 23 AND s.collected_at >= 90 AND s.collected_at <= 160 ORDER BY collected_at ASC",
   "SELECT * FROM a ORDER BY x ASC NULLS FIRST",
   "SELECT * FROM a ORDER BY x ASC NULLS LAST",
   "SELECT * FROM a ORDER BY x COLLATE \"tr_TR\" DESC NULLS LAST",
@@ -54,20 +54,20 @@ const char* tests[] = {
   "SELECT * FROM x WHERE id IN (SELECT id FROM account)",
   "SELECT * FROM x WHERE id NOT IN (1, 2, 3)",
   "SELECT * FROM x JOIN (SELECT n FROM z) b ON a.id = b.id",
-  "SELECT * FROM x WHERE y = z[?]",
+  "SELECT * FROM x WHERE y = z[5]",
   "SELECT (foo(1)).y",
   "SELECT proname, (SELECT regexp_split_to_array(proargtypes::text, ' '))[idx] AS argtype, proargnames[idx] AS argname FROM pg_proc",
   "SELECT COALESCE((SELECT customer.sp_person(n.id) AS sp_person).city_id, NULL::int) AS city_id FROM customer.tb_customer n",
-  "SELECT * FROM x WHERE y = z[?][?]",
+  "SELECT * FROM x WHERE y = z[1][4]",
   "SELECT (k #= hstore('{id}'::text[], ARRAY[1::text])).* FROM test k",
   "SELECT * FROM x WHERE NOT y",
   "SELECT * FROM x WHERE x OR y",
   "SELECT 1 WHERE (1 = 1 OR 1 = 2) AND 1 = 2",
   "SELECT 1 WHERE (1 = 1 AND 2 = 2) OR 2 = 3",
   "SELECT 1 WHERE 1 = 1 OR 2 = 2 OR 2 = 3",
-  "SELECT * FROM x WHERE x = ALL(?)",
-  "SELECT * FROM x WHERE x = ANY(?)",
-  "SELECT * FROM x WHERE x = COALESCE(y, ?)",
+  "SELECT * FROM x WHERE x = ALL (SELECT x FROM y WHERE z = 20)",
+  "SELECT * FROM x WHERE x = ANY (SELECT x FROM y WHERE z = 20)",
+  "SELECT * FROM x WHERE x = COALESCE(y, z)",
   "SELECT a, b, max(c) FROM c WHERE d = 1 GROUP BY a, b",
   "SELECT * FROM x LIMIT 50",
   "SELECT * FROM x OFFSET 50",
@@ -89,12 +89,11 @@ const char* tests[] = {
   "SELECT row_cols[0] AS dept, row_cols[1] AS sub, admin, ordinary FROM crosstab('SELECT ARRAY[department, sub] AS row_cols, role, COUNT(id) FROM users GROUP BY department, role ORDER BY department, role', 'VALUES (''admin''::text), (''ordinary''::text)') AS (row_cols varchar[], admin int, ordinary int)",
   "SELECT 1::int8",
   "SELECT CAST(1 + 3 AS int8)",
-  "SELECT ?::regclass",
+  "SELECT x::regclass",
   "SELECT table_field::bool, table_field::boolean FROM t",
   "SELECT true, false",
   "SELECT 1::boolean, 0::boolean",
   "SELECT $5",
-  "SELECT ?",
   "INSERT INTO x (y, z) VALUES (1, 'abc')",
   "INSERT INTO x (\"user\") VALUES ('abc')",
   "INSERT INTO x (y, z) VALUES (1, 'abc') RETURNING id",
@@ -117,7 +116,7 @@ const char* tests[] = {
   "WITH archived AS (DELETE FROM employees WHERE manager_name = 'Mary') UPDATE users SET archived = true WHERE users.id IN (SELECT user_id FROM moved)",
   "WITH archived AS (DELETE FROM employees WHERE manager_name = 'Mary' RETURNING user_id) UPDATE users SET archived = true FROM archived WHERE archived.user_id = id RETURNING id",
   "INSERT INTO jackdanger_card_totals (id, amount_cents, created_at) SELECT series.i, random() * 1000, (SELECT '2015-08-25 00:00:00 -0700'::timestamp + (('2015-08-25 23:59:59 -0700'::timestamp - '2015-08-25 00:00:00 -0700'::timestamp) * random())) FROM generate_series(1, 10000) series(i)",
-  "UPDATE foo SET a = ?, b = ?",
+  "UPDATE foo SET a = 24, b = true",
   "UPDATE x SET \"user\" = 'emin'",
   "DELETE FROM x WHERE y = 1",
   "DELETE FROM ONLY x table_x USING table_z WHERE y = 1 RETURNING *",
@@ -137,7 +136,7 @@ const char* tests[] = {
   "CREATE SCHEMA IF NOT EXISTS test AUTHORIZATION joe",
   "CREATE SCHEMA hollywood CREATE TABLE films (title text, release date, awards text[]) CREATE VIEW winners AS SELECT title, release FROM films WHERE awards IS NOT NULL",
   "CREATE UNLOGGED TABLE cities (name text, population real, altitude double, identifier smallint, postal_code int, foreign_id bigint)",
-  "CREATE TABLE IF NOT EXISTS distributors (name varchar(40) DEFAULT 'Luso Films', len interval hour to second(3), name varchar(40) DEFAULT 'Luso Films', did int DEFAULT nextval('distributors_serial'), stamp timestamp DEFAULT now() NOT NULL, stamptz timestamp with time zone, time time NOT NULL, timetz time with time zone, CONSTRAINT name_len PRIMARY KEY (name, len))",
+  "CREATE TABLE IF NOT EXISTS distributors (name varchar(40) DEFAULT 'Luso Films', len interval hour to second(3), name varchar(40) DEFAULT 'Luso Films', did int DEFAULT nextval('distributors_serial'), stamp timestamp DEFAULT now() NOT NULL, stamptz timestamp with time zone, \"time\" time NOT NULL, timetz time with time zone, CONSTRAINT name_len PRIMARY KEY (name, len))",
   "CREATE TABLE types (a real, b double precision, c numeric(2, 3), d char(4), e char(5), f varchar(6), g varchar(7))",
   "CREATE TABLE types (a geometry(point) NOT NULL)",
   "CREATE TABLE tablename (colname int NOT NULL DEFAULT nextval('tablename_colname_seq'))",
@@ -222,7 +221,7 @@ const char* tests[] = {
   "SET statement_timeout TO 10000",
   "SET search_path TO my_schema, public",
   "SET LOCAL search_path TO my_schema, public",
-  "SET \"user\" TO ?",
+  "SET \"user\" TO 4",
   "VACUUM",
   "VACUUM t",
   "VACUUM (FULL) t",
@@ -382,8 +381,12 @@ const char* tests[] = {
   "CREATE EXTENSION x",
   "CREATE EXTENSION IF NOT EXISTS x CASCADE VERSION \"1.2\" SCHEMA a",
   "CREATE TABLE like_constraint_rename_cache (LIKE constraint_rename_cache INCLUDING ALL)",
-  "COPY manual_export TO STDOUT WITH (FORMAT CSV, HEADER)",
-  "SELECT 1 FROM a.b.c"
+  "COPY manual_export TO STDOUT CSV HEADER",
+  "SELECT 1 FROM a.b.c",
+  "CREATE PUBLICATION foo FOR TABLES IN SCHEMA bar",
+  "ALTER TABLE ALL IN TABLESPACE foo OWNED BY bar, quux SET TABLESPACE fred NOWAIT",
+  "MERGE INTO measurement m USING new_measurement nm ON m.city_id = nm.city_id AND m.logdate = nm.logdate WHEN MATCHED AND nm.peaktemp IS NULL THEN DELETE WHEN MATCHED THEN UPDATE SET peaktemp = GREATEST(m.peaktemp, nm.peaktemp), unitsales = m.unitsales + COALESCE(nm.unitsales, 0) WHEN NOT MATCHED THEN INSERT (city_id, logdate, peaktemp, unitsales) VALUES (city_id, logdate, peaktemp, unitsales)",
+  "COPY vistest FROM STDIN FREEZE CSV",
 };
 
 size_t testsLength = __LINE__ - 4;
