@@ -523,3 +523,13 @@ BEGIN
 
   RETURN 1;
 END;$$;
+
+-- Example from https://www.postgresql.org/docs/current/sql-do.html
+DO $$DECLARE r record;
+BEGIN
+    FOR r IN SELECT table_schema, table_name FROM information_schema.tables
+             WHERE table_type = 'VIEW' AND table_schema = 'public'
+    LOOP
+        EXECUTE 'GRANT ALL ON ' || quote_ident(r.table_schema) || '.' || quote_ident(r.table_name) || ' TO webuser';
+    END LOOP;
+END$$;
\ No newline at end of file
